﻿Public Class GPSdata_class
    Structure posblh
        Dim lat As Double '緯度
        Dim lon As Double '経度
        Dim hgt As Double '高度
    End Structure
    Structure posxyz
        Dim x As Double
        Dim y As Double
        Dim z As Double
    End Structure
    '地球に関するパラメータ
    Public Const equatorial_radius As Double = 6378137.0                            ' 赤道半径[m]    WGS-84測地系による
    Public Const invers_oblateness As Double = 298.257223                           ' 扁平率の逆数   WGS-84測地系による
    Public Const oblateness As Double = 1 / invers_oblateness                       ' 扁平率
    Public Const short_radius As Double = equatorial_radius * (1 - oblateness)      ' 短半径
    Public Const square_eccentricity As Double = oblateness * (2.0 - oblateness)    ' 離心率^2
    Public Function xyz_to_blh(ByVal pos As posxyz) As posblh
        Dim posblh_temp As posblh
        Dim a, b, e, f, n, h, p, t, sint, cost As Double

        posblh_temp.lat = 0
        posblh_temp.lon = 0
        posblh_temp.hgt = -equatorial_radius

        If pos.x = 0 And pos.y = 0 And pos.z = 0 Then Return posblh_temp

        f = oblateness                      ' 扁平率
        a = equatorial_radius               ' 長半径
        b = short_radius                    ' 短半径
        e = Math.Sqrt(square_eccentricity)  ' 離心率

        '　座標変換のためのパラメータ
        h = a ^ 2 - b ^ 2
        p = Math.Sqrt(pos.x ^ 2 + pos.y ^ 2)
        t = Math.Atan2(pos.z * a, p * b)
        sint = Math.Sin(t)
        cost = Math.Cos(t)

        posblh_temp.lat = Math.Atan2(pos.z + h / b * sint ^ 3, p - h / a * cost ^ 3)    ' 緯度[rad]を計算する
        n = a / Math.Sqrt(1 - square_eccentricity * Math.Sin(posblh_temp.lat) ^ 2)      ' 卯酉線曲率半径
        posblh_temp.lon = Math.Atan2(pos.y, pos.x)                                      ' 経度[rad]を求める
        posblh_temp.hgt = (p / Math.Cos(posblh_temp.lat)) - n                           ' 楕円体高[m]

        Return posblh_temp
    End Function

End Class
